# -*- coding: utf-8 -*-

import datetime
import json

import click

from em_celery import logger, init_db
from em_tasks.utils.price_calculator import PriceCalculator
from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.order_util import OrderUtil
from em_tasks.utils.pipelines.price_calculator import PriceCalculatorPipeline
from em_tasks.utils.pipelines.sku_updater import SkuUpdaterPipeline
from em_tasks.utils.pipelines.image_url_corrector import ImageUrlCorrectorPipeline
from em_tasks.utils.pipelines.amz_products_filter import AmzProductsFilterPipeline
from em_tasks.utils.pipelines.upc_validator import UpcValidatorPipeline
from em_tasks.utils.product_loaders.amz_es_product_loader import AmzEsProductLoader
from em_tasks.utils.offer_services import AmzOfferService
from em_tasks.store.models import *

from em_celery import logger, get_config, init_db, get_product_service, get_offer_service_config, get_amz_offer_filter_config


@click.command('Format products to Spree style')
@click.option('-s', '--source', required=True, type=str, help='Product source.')
@click.option('-sc', '--source_code', required=True, type=str, help='Product source code.')
@click.option('-sp', '--sku_prefix', type=str, default='', help='Current product sku prefix to replace.')
@click.option('-o', '--output_path', required=True, type=str)
@click.option('-m', '--marketplace', type=str)
@click.argument('products_path')
def format_products(products_path, output_path, source, source_code, marketplace, sku_prefix=''):
  init_db()

  if not products_path or not output_path:
    return

  products_path = os.path.abspath(os.path.expanduser(products_path))
  if not os.path.isfile(products_path):
    return

  output_path = os.path.abspath(os.path.expanduser(output_path))
  output_dir = os.path.dirname(output_path)
  if not os.path.isdir(output_dir):
    os.makedirs(output_dir)

  price_rules = {
    'roi': 0.3,
    'ad_cost': 5,
    'transfer_cost': 12
  }

  cfg = get_config()
  price_rules_cfg = cfg.get('price.rules.{}'.format(source.lower()), dict())
  for k, v in price_rules_cfg.items():
    price_rules[k] = round(float(v), 2)
  if 'min_profit_amount' in price_rules:
    min_profit_amount = price_rules['min_profit_amount']
  else:
    min_profit_amount = 10
  price_calculator = PriceCalculator(price_rules, 'USD', min_profit_amount)
  logger.debug('[PriceRules] %s', price_rules)

  product_service = get_product_service()
  filter_cond = get_amz_offer_filter_config()
  offer_service_cfg = get_offer_service_config()
  offer_service = AmzOfferService(
    offer_service_cfg['host'], offer_service_cfg['port'],
    offer_service_cfg['user'], offer_service_cfg['password'], filter_cond)

  if not marketplace:
    marketplace = source.split('_').pop().lower()

  price_calculator_pipeline = PriceCalculatorPipeline(price_calculator)
  sku_updater_pipeline = SkuUpdaterPipeline(sku_prefix, source_code)
  image_url_corrector_pipeline = ImageUrlCorrectorPipeline()
  products_filter_pipeline = AmzProductsFilterPipeline()
  upc_validator_pipeline = UpcValidatorPipeline()
  pipelines = [products_filter_pipeline, price_calculator_pipeline, sku_updater_pipeline, image_url_corrector_pipeline, upc_validator_pipeline]

  products_loader = AmzEsProductLoader(product_service, offer_service, marketplace, products_path)
  with open(output_path, 'w+', encoding='utf-8', errors='ignore') as output_fh:
    for product in products_loader.load():
      for pipeline in pipelines:
        product = pipeline.execute(product)
        if not product:
          break

      if not product or not product.get('price', 0):
        continue
      product["shipping_days_min"] = None
      product["shipping_days_max"] = None
      output_fh.write(json.dumps(product, ensure_ascii=False))
      output_fh.write("\n")


if __name__ == "__main__":
  format_products()
